﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media.Imaging;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace CSHowDoISamples
{
    public partial class AddSimpleMarkers : UserControl
    {
        private bool _isInitialized = false;

        public AddSimpleMarkers()
        {
            InitializeComponent();
        }

        private void WpfMap_Loaded(object sender, RoutedEventArgs e)
        {
            if (_isInitialized) return;

            wpfMap1.MapUnit = GeographyUnit.DecimalDegree;
            wpfMap1.CurrentExtent = new RectangleShape(-155.733, 95.60, 104.42, -81.9);

            WorldMapKitWmsWpfOverlay worldMapKitOverlay = new WorldMapKitWmsWpfOverlay();
            wpfMap1.Overlays.Add(worldMapKitOverlay);

            SimpleMarkerOverlay markerOverlay = new SimpleMarkerOverlay();
            markerOverlay.DragMode = MarkerDragMode.Drag;

            wpfMap1.Overlays.Add("MarkerOverlay", markerOverlay);
            wpfMap1.Refresh();

            _isInitialized = true;
        }

      
        private void wpfMap1_MapClick(object sender, MapClickWpfMapEventArgs e)
        {
            if (e.MouseButton != MapMouseButton.Left) 
                return;

            SimpleMarkerOverlay markerOverlay = (SimpleMarkerOverlay) wpfMap1.Overlays["MarkerOverlay"];
            AddPictureMarkerToOverlay(e.WorldLocation, markerOverlay);
            markerOverlay.Refresh();
        }

        private void OnMarkerPreviewMouseRightButtonUp(object sender, MouseButtonEventArgs e)
        {
            ((Marker)sender).ContextMenu.IsOpen = true;
        }

        private void AddPictureMarkerToOverlay(PointShape point, SimpleMarkerOverlay markerOverlay)
        {
            Marker marker = new Marker(point);
            marker.ImageSource = new BitmapImage(new Uri("/Resources/AQUA.png", UriKind.RelativeOrAbsolute));
            marker.Width = 32;
            marker.Height = 32;
            marker.YOffset = -17;

            CompleteMarkerSetup(marker);

            markerOverlay.Markers.Add(marker.Name, marker);
        }

        private void CompleteMarkerSetup(Marker marker)
        {
            var contextMenu = new ContextMenu();
            var menuItem = new MenuItem();
            menuItem.Header = "Remove";
            menuItem.Name = "Marker" + marker.GetHashCode().ToString();
            menuItem.Click += OnRemoveMarkerRequested;
            contextMenu.Items.Add(menuItem);

            marker.ContextMenu = contextMenu;
            marker.Name = menuItem.Name;
            marker.PreviewMouseRightButtonUp += OnMarkerPreviewMouseRightButtonUp;
        }

        private void OnRemoveMarkerRequested(object sender, RoutedEventArgs e)
        {
            var markerOverlay = (SimpleMarkerOverlay)wpfMap1.Overlays["MarkerOverlay"];
            var markerName = ((MenuItem) e.Source).Name;
            markerOverlay.Markers.Remove(markerName);
            markerOverlay.Refresh();
        }
    }
}
